import QtQuick 2.4
import Process 0.1
import "../"

// Keypad.qml - Keypad control for entering numeric information.

Item {
    id: root

    signal valueEntered (string val)
    signal keyPress(string key)
    signal close()
    signal multipress(variant keylist)

    property variant keys_model: state === "lowercase" ? lowercase_keys : uppercase_keys
    property int length:20
    property string keymap: ""
    
    // Space between keys
    readonly property real keySpacing:   5*main.hScaleFactor
    // Height of key
    readonly property real keyHeight:    65*main.vScaleFactor
    // Width of a 1-unit key
    readonly property real keyWidth1u:   72*main.hScaleFactor
    // Calculate these common widths
    readonly property real keyWidth1_5u: getKeyWidth(1.5)
    readonly property real keyWidth2u:   getKeyWidth(2)
    readonly property real keyWidth3u:   getKeyWidth(3)
    
    /* Widths of keys wider than 1u; for a(n) X-unit key, the width
     *  will be: X*keyWidth1u+(X-1)*keySpacing 
     * (This could probably be used for X > 0, too...) */
    function getKeyWidth(w) {
        var k = (w >= 1) ? w : 1;
        return k*keyWidth1u+(k-1)*keySpacing;
    }

    width: keypad_box.width
    height: childrenRect.height

    state: getState()

    function getState() {
        // Get initial state depending on language
        if (application.currentLanguage === "ar") {
            return "ArabicLowercase"
        }
        return "lowercase"
    }

    function setKeymap(keymapIn) {
        var keymap = "us"

        if (keymapIn) {
            keymap = keymapIn
        }
        console.log("Setting keymap to", keymap)
        setxkbmap.arguments = ["-layout", keymap]
        setxkbmap.start()
        root.keymap = keymap
    }

    function show() {
        root.visible = true
    }

    function hide() {
        root.visible = false
    }

    function keyFunction(val) {
        print("key function: " + val)
    }

    Connections {
        target: application
        onCurrentLanguageChanged: {
            // Reset state
            root.state = getState()
        }
        // Need to reset state any time keypad is reshown
        // Another keypad could have called setxkbmap and put this keypad out of sync w/ the global keymap
        onWindowShown: {
            console.log("window shown...")
            root.state = ""
            root.state = getState()
        }
    }

    Process {
        id: setxkbmap

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            console.log(output)
        }
        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            console.log(output)
        }

        program: "/usr/bin/setxkbmap"
    }

    // stack top entry box on top of keypad

    // main keypad body
    Item {
        id: keypad_box

        visible: true
        anchors.top: parent.top
        //anchors.topMargin: -37
        anchors.horizontalCenter: parent.horizontalCenter
        x: 10*main.hScaleFactor
        y: 90*main.vScaleFactor
        width: (langButton.visible ? 865 : 830)*main.hScaleFactor
        height: 300*main.vScaleFactor
        anchors.horizontalCenterOffset: 0

        MouseArea {
            anchors.fill: parent
            onClicked: {} // Block clicks from affecting items underneath
        }

        KeyButton {
            id: xButton
            
            anchors { top: parent.top; right: parent.right; rightMargin: 2*main.hScaleFactor; topMargin: 2*main.vScaleFactor }
            height: root.keyHeight/2
            width: root.keyWidth1u/2
            
            color: main.colorBackground
            border_color: main.colorBorder
            border_width: 2*main.scaleFactor
            
            label: "X"
            label_color: "white"
            label_visible: true
            fontsize: height*0.75*main.vScaleFactor

            enabled: true
            onClicked: {
                root.close()
            }
        }

        LanguageSelector {
            id: langButton

            visible: application.currentLanguage === "zh" || application.currentLanguage === "ja"
            anchors {
                bottom: parent.bottom
                bottomMargin: 0
                right: parent.right
                rightMargin: 1
            }
        }

        Column {
            id: keypad_column
            anchors { left: parent.left; top: parent.top }
            anchors.topMargin: 10*main.vScaleFactor
            anchors.leftMargin: 15*main.hScaleFactor
            anchors.rightMargin: 15*main.hScaleFactor
            width: 800*main.hScaleFactor
            spacing: 0
            Flow {
                id: keygrid
                width: parent.width
                spacing: 5*main.hScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter

                Repeater {
                    model: keys_model
                    delegate: keypad_button
                }
            }
        }
    }
    
    // delegate for keypad button
    Component {
        id: keypad_button
        
        KeyButton {
            height: root.keyHeight
            width: (model.keywidth === 1) ? root.keyWidth1u : (model.keywidth === 2) ? root.keyWidth2u : (model.keywidth === 3) ? root.keyWidth3u : root.keyWidth1_5u
            
            color: main.colorBackground
            border_color: main.colorBorder
            border_width: 2*main.hScaleFactor
            
            icon: image ? image : ""
            label: size ? model.label : ""
            label_color: "white"
            label_visible: (application.currentLanguage !== "en" && value === "KP_Enter") ? false : true
            fontsize: size*main.vScaleFactor
            label_x_offset: x_off
            label_y_offset: y_off

            enabled: true
            onClicked: {
                if(func) {
                    print(func)
                    eval(func)
                }
                else
                    keyPress(value)
            }
        }
    }

    states: [
        // States for Arabic
        State {
            name: "ArabicUppercase"
            PropertyChanges { target: root ; keys_model: arabic_uppercase }
            PropertyChanges { target: keypad_box; width: 980*main.hScaleFactor }
            PropertyChanges { target: keypad_column; width: 950*main.hScaleFactor }
            StateChangeScript { script: root.setKeymap("ar") }
        },
        State {
            name: "ArabicLowercase"
            PropertyChanges { target: root; keys_model: arabic_lowercase }
            PropertyChanges { target: keypad_box; width: 980*main.hScaleFactor }
            PropertyChanges { target: keypad_column; width: 950*main.hScaleFactor }
            StateChangeScript { script: root.setKeymap("ar") }
        },
        State {
            name: "ArabicAlpha"
            PropertyChanges { target: root; keys_model: ar_lowercase_keys }
            PropertyChanges { target: keypad_box; width: 830*main.hScaleFactor }
            PropertyChanges { target: keypad_column; width: 800*main.hScaleFactor }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "ArabicAlphaUpper"
            PropertyChanges { target: root; keys_model: ar_uppercase_keys }
            PropertyChanges { target: keypad_box; width: 830*main.hScaleFactor }
            PropertyChanges { target: keypad_column; width: 800*main.hScaleFactor }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "ArabicNumeric"
            PropertyChanges { target: root; keys_model: ar_number_keys }
            PropertyChanges { target: keypad_box; width: 830*main.hScaleFactor }
            PropertyChanges { target: keypad_column; width: 800*main.hScaleFactor }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "ArabicSymbols"
            PropertyChanges { target: root; keys_model: ar_symbol_keys }
            PropertyChanges { target: keypad_box; width: 830*main.hScaleFactor }
            PropertyChanges { target: keypad_column; width: 800*main.hScaleFactor }
            StateChangeScript { script: root.setKeymap("us") }
        },


        State {
            name: "uppercase"
            PropertyChanges { target: root ; keys_model: uppercase_keys }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "lowercase"
            PropertyChanges { target: root ; keys_model: lowercase_keys }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "numbers"
            PropertyChanges { target: root ; keys_model: number_keys }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "symbols"
            PropertyChanges { target: root ; keys_model: symbol_keys }
            StateChangeScript { script: root.setKeymap("us") }
        }
    ]

    // model of keypad keys
    // The `value` column comes from X11/keysymdef.h as this is what must be passed to xdotool
    ListModel {
        id: uppercase_keys

        ListElement { label: "Q" ; value: "Q"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "W"; value: "W"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "E"; value: "E"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "R"; value: "R"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "T"; value: "T"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "Y"; value: "Y"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "U"; value: "U"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "I"; value: "I"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "O"; value: "O"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "P"; value: "P"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "A" ; value: "A"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "S"; value: "S"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "D"; value: "D"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "F"; value: "F"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "G"; value: "G"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "H"; value: "H"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "J"; value: "J"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "K"; value: "K"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "L"; value: "L"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "" ; value: ""; keywidth: 1; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='lowercase'"}
        ListElement { label: "Z"; value: "Z"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "X"; value: "X"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "C"; value: "C"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "V"; value: "V"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "B"; value: "B"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "N"; value: "N"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "M"; value: "M"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}
        ListElement { label: "?123"; value: ""; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='numbers'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }

    ListModel {
        id: lowercase_keys

        ListElement { label: "q"; value: "q"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "w"; value: "w"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "e"; value: "e"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "r"; value: "r"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "t"; value: "t"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "y"; value: "y"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "u"; value: "u"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "i"; value: "i"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "o"; value: "o"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "p"; value: "p"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "a"; value: "a"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "s"; value: "s"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "d"; value: "d"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "f"; value: "f"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "g"; value: "g"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "h"; value: "h"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "j"; value: "j"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "k"; value: "k"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "l"; value: "l"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: ""; value: ""; keywidth: 1; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='uppercase'"}
        ListElement { label: "z"; value: "z"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "x"; value: "x"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "c"; value: "c"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "v"; value: "v"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "b"; value: "b"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "n"; value: "n"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "m"; value: "m"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "?123"; value: "?123"; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='numbers'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }

    // Based on IBM 101-key Arabic keyboard
    // See <ftp://ftp.software.ibm.com/software/globalization/keyboards/KBD470.pdf>
    ListModel {
        id: arabic_lowercase

        ListElement { label: "\uFEBF"; value: "Arabic_dad"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEBB"; value: "Arabic_sad"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE9B"; value: "Arabic_theh"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFED7"; value: "Arabic_qaf"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFED3"; value: "Arabic_feh"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFECF"; value: "Arabic_ghain"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFECB"; value: "Arabic_ain"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEEB"; value: "Arabic_heh"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEA7"; value: "Arabic_khah"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEA3"; value: "Arabic_hah"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE9F"; value: "Arabic_jeem"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEA9"; value: "Arabic_dal"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "\uFEB7"; value: "Arabic_sheen"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEB3"; value: "Arabic_seen"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEF3"; value: "Arabic_yeh"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE91"; value: "Arabic_beh"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEDF"; value: "Arabic_lam"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE8D"; value: "Arabic_alef"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE97"; value: "Arabic_teh"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEE7"; value: "Arabic_noon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEE3"; value: "Arabic_meem"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEDB"; value: "Arabic_kaf"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEC3"; value: "Arabic_tah"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: ""; keywidth: 1; size: 45; x_off: 0; y_off: 0; func: ""} // empty key

        ListElement { label: ""; value: ""; keywidth: 1; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicUppercase'"}
        ListElement { label: "\uFE8B"; value: "Arabic_hamzaonyeh"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE80"; value: "Arabic_hamza"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE85"; value: "Arabic_hamzaonwaw"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEAD"; value: "Arabic_ra"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEFB"; value: "0x100fefb"; keywidth: 1; size: 45; x_off: 0; y_off: 0} // TODO should this be 0x100fefb? - see "dist/symbols/ara"
        ListElement { label: "\uFEEF"; value: "Arabic_alefmaksura"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE93"; value: "Arabic_tehmarbuta"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEED"; value: "Arabic_waw"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEAF"; value: "Arabic_zain"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEC7"; value: "Arabic_zah"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlpha'"}
        ListElement { label: "?123"; value: "?123"; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }

    ListModel {
        id: arabic_uppercase
        ListElement { label: "\u064E"; value: "Arabic_fatha"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064B"; value: "Arabic_fathatan"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064F"; value: "Arabic_damma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064C"; value: "Arabic_dammatan"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: "??"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0625"; value: "Arabic_hamzaunderalef"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0060"; value: "grave"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u00F7"; value: "division"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u00D7"; value: "multiply"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u061B"; value: "Arabic_semicolon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        /*Less Than*/ListElement { label: "<"; value: "Arabic_semicolon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        /*Greater Than*/ListElement { label: ">"; value: "Arabic_semicolon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "\u0650"; value: "Arabic_kasra"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064D"; value: "Arabic_kasratan"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "]"; value: "bracketright"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "["; value: "bracketleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: "0x100fef7"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0623"; value: "Arabic_hamzaonalef"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0640"; value: "Arabic_tatweel"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u060C"; value: "Arabic_comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ":"; value: "colon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\""; value: "quotedbl"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: ""; value: ""; keywidth: 1; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "~"; value: "asciitilde"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0652"; value: "Arabic_sukun"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "}"; value: "braceright"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "{"; value: "braceleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: "0x100fef5"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0622"; value: "Arabic_maddaonalef"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "'"; value: "apostrophe"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u061F"; value: "Arabic_question_mark"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlpha'"}
        ListElement { label: "?123"; value: "?123"; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_lowercase_keys

        ListElement { label: "q"; value: "q"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "w"; value: "w"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "e"; value: "e"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "r"; value: "r"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "t"; value: "t"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "y"; value: "y"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "u"; value: "u"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "i"; value: "i"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "o"; value: "o"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "p"; value: "p"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "a"; value: "a"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "s"; value: "s"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "d"; value: "d"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "f"; value: "f"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "g"; value: "g"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "h"; value: "h"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "j"; value: "j"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "k"; value: "k"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "l"; value: "l"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: ""; value: ""; keywidth: 1; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlphaUpper'"}
        ListElement { label: "z"; value: "z"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "x"; value: "x"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "c"; value: "c"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "v"; value: "v"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "b"; value: "b"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "n"; value: "n"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "m"; value: "m"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; keywidth: 1.5; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "?#"; value: "?123"; keywidth: 1.5; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; keywidth: 1; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_uppercase_keys

        ListElement { label: "Q" ; value: "Q"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "W"; value: "W"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "E"; value: "E"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "R"; value: "R"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "T"; value: "T"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "Y"; value: "Y"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "U"; value: "U"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "I"; value: "I"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "O"; value: "O"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "P"; value: "P"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "A" ; value: "A"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "S"; value: "S"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "D"; value: "D"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "F"; value: "F"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "G"; value: "G"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "H"; value: "H"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "J"; value: "J"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "K"; value: "K"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "L"; value: "L"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "" ; value: ""; keywidth: 1; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlpha'"}
        ListElement { label: "Z"; value: "Z"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "X"; value: "X"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "C"; value: "C"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "V"; value: "V"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "B"; value: "B"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "N"; value: "N"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "M"; value: "M"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; keywidth: 1.5; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "?#"; value: ""; keywidth: 1.5; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; keywidth: 1; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_number_keys

        ListElement { label: "1" ; keywidth: 1; value: "1" ; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "2"; value: "2"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "3"; value: "3"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "4"; value: "4"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "5"; value: "5"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "6"; value: "6"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "7"; value: "7"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "8"; value: "8"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "9"; value: "9"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "0"; value: "0"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "-" ; value: "minus"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ":"; value: "colon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "("; value: "parenleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ")"; value: "parenright"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: 'quotedbl' ;keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "#+="; value: "" ; keywidth: 1;  size: 30; x_off: 0; y_off: 0 ; func: "root.state='ArabicSymbols'"}
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ;image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_symbol_keys

        ListElement { label: "]"; value: "bracketright"; keywidth: 1;  size: 45; x_off: 0; y_off: 0}
        ListElement { label: "["; value: "bracketleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "{"; value: "braceleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "}"; value: "braceright"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "%"; value: "percent"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "^"; value: "asciicircum"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "+"; value: "plus"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "_" ;keywidth: 1; value: "underscore" ; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\\"; value: "backslash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "|"; value: "bar"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "~"; value: "asciitilde"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<"; value: "less"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ">"; value: "greater"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: "quotedbl" ;keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "123" ; value: "" ; keywidth: 1;  size: 30; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ; image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }

    // Default - English
    ListModel {
        id: number_keys

        ListElement { label: "1"; value: "1"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "2"; value: "2"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "3"; value: "3"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "4"; value: "4"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "5"; value: "5"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "6"; value: "6"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "7"; value: "7"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "8"; value: "8"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "9"; value: "9"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "0"; value: "0"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "-" ; value: "minus"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ":"; value: "colon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "("; value: "parenleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ")"; value: "parenright"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: 'quotedbl' ;keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "#+="; value: "" ; keywidth: 1;  size: 30; x_off: 0; y_off: 0 ; func: "root.state='symbols'"}
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='lowercase'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ;image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }

    ListModel {
        id: symbol_keys

        ListElement { label: "]"; value: "bracketright"; keywidth: 1;  size: 45; x_off: 0; y_off: 0}
        ListElement { label: "["; value: "bracketleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "{"; value: "braceleft"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "}"; value: "braceright"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "%"; value: "percent"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "^"; value: "asciicircum"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "+"; value: "plus"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "_" ;keywidth: 1; value: "underscore" ; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\\"; value: "backslash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "|"; value: "bar"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "~"; value: "asciitilde"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<"; value: "less"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ">"; value: "greater"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: "quotedbl" ;keywidth: 1; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "123" ; value: "" ; keywidth: 1;  size: 30; x_off: 0; y_off: 0 ; func: "root.state='numbers'"}
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; keywidth: 1; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; keywidth: 2; size: 45; x_off: 0; y_off: 0 ; func: "root.state='lowercase'"}
        ListElement { label: "SPACE"; value: "space"; keywidth: 3; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; keywidth: 1; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; keywidth: 1; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ; image: "Keypad/keypad_returnicon.png"; keywidth: 2; size: 14; x_off: 0; y_off: -10}
    }


}
